#ifndef __COMPAT_LINUX_LIST_LRU_H__
#define __COMPAT_LINUX_LIST_LRU_H__

#include_next <linux/list_lru.h>

#ifdef COMPAT_DETECT_LIST_LRU_ADD_OBJ_6_8
#define list_lru_add list_lru_add_obj
#endif

#ifdef COMPAT_DETECT_LIST_LRU_DEL_OBJ_6_8
#define list_lru_del list_lru_del_obj
#endif

#endif /* __COMPAT_LINUX_LIST_LRU_H__ */
