/* Dropdown Button */
.dropbtn {
  padding: 16px;
  font-size: 16px;
  color: var(--fg);
  border: none;
  background-color: var(--bg);
}

/* The container <div> - needed to position the dropdown content */
.dropdown {
  position: relative;
  display: inline-block;
  color: var(--fg);
  background-color: var(--bg);
}

/* Dropdown Content (Hidden by Default) */
.dropdown-content {
  display: none;
  position: absolute;
  background-color: var(--bg);
  background: var(--theme-popup-bg);
  border: 1px solid var(--theme-popup-border);
  color: var(--fg);
  z-index: 1;
}

/* Links inside the dropdown */
.dropdown-content a {
  color: var(--fg);
  padding: 12px 16px;
  text-decoration: none;
  display: block;
}

/* Change color of dropdown links on hover */
.dropdown-content a:hover {background-color: var(--theme-hover);}

/* Show the dropdown menu on hover */
.dropdown:hover .dropdown-content {display: block;}

/* Change the background color of the dropdown button when the dropdown content is shown */
/* .dropdown:hover .dropbtn {background-color: #3e8e41;} */
