#ifndef __COMPAT_LINUX_SYSCTL_H__
#define __COMPAT_LINUX_SYSCTL_H__

#include_next <linux/sysctl.h>

#ifdef COMPAT_DETECT_STRUCT_CTL_TABLE_6_11

#define COMPAT_SYSCTL_TABLE_CONST 	const
#define COMPAT_SYSCTL_TABLE_TERMINATOR

#else /* !COMPAT_DETECT_STRUCT_CTL_TABLE_6_11 */

#define COMPAT_SYSCTL_TABLE_CONST
#define COMPAT_SYSCTL_TABLE_TERMINATOR	{ }

#endif /* COMPAT_DETECT_STRUCT_CTL_TABLE_6_11 */

#endif /* __COMPAT_LINUX_SYSCTL_H__ */
