#include <linux/module.h>
#include <linux/pagemap.h>

void *test_dummy(void)
{
	/* 
	 * v6.5-rc1~176^2~213 - "filemap: add a kiocb_invalidate_pages helper"
	 *
	 * Though what we really want is 82c25e9c157f37bd0ab5a82fe2417e2223df459
	 * "filemap: update ki_pos in generic_perform_write"
	 *
	 * But it exposes no APIs.
	 */
	return (void *)&kiocb_invalidate_pages;
}

MODULE_LICENSE("GPL");
