#ifndef __COMPAT_LINUX_DCACHE_H__
#define __COMPAT_LINUX_DCACHE_H__

#include_next <linux/dcache.h>

#ifndef COMPAT_DETECT_D_HASH_AND_LOOKUP_6_16
extern struct dentry *d_hash_and_lookup(struct dentry *dir, struct qstr *name);
#endif /* COMPAT_DETECT_D_HASH_AND_LOOKUP_6_16 */

#ifndef COMPAT_DETECT_SET_DEFAULT_D_OP_6_17
#define set_default_d_op(sb, ops) sb->s_d_op = ops
#endif

#endif /* __COMPAT_LINUX_DCACHE_H__ */
