#include <linux/module.h>
#include <linux/fs.h>
#include <linux/mm.h>

void check_type(const struct address_space_operations *aops, struct file *f, struct address_space *ao,
                struct list_head *l, unsigned int flags)
{
	/* v5.18-rc1~64^2~89 */
	aops->readpages(f, ao, l, flags);
}

MODULE_LICENSE("GPL");
