#ifndef __COMPAT_LINUX_EXPORTFS_H__
#define __COMPAT_LINUX_EXPORTFS_H__

#include_next <linux/exportfs.h>

#ifndef COMPAT_DETECT_EXPORT_OP_FLUSH_ON_CLOSE_6_4
#define EXPORT_OP_FLUSH_ON_CLOSE 0
#endif

#endif /* __COMPAT_LINUX_EXPORTFS_H__ */
